﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Runtime.Serialization.Formatters.Binary;

namespace AltitudeShaderGen
{
    public partial class FrmMain : Form
    {
        public FrmMain()
        {
            InitializeComponent();
            AddLC();
        }

        private void btnAdd_Click(object sender, EventArgs e)
        {
            AddLC();
        }

        private void AddLC(LevelControl lc = null)
        {
            if (lc == null)
            {
                lc = new LevelControl();
            }
            lc.Remove += new LevelControl.RemoveHandler(lc_Remove);
            flowLayoutPanel.Controls.Add(lc);
        }

        void lc_Remove(LevelControl levelControl)
        {
            flowLayoutPanel.Controls.Remove(levelControl);
        }

        const string SL_BASE_NAME = "AltitudeLevel ";

        private void btnGenerate_Click(object sender, EventArgs e)
        {            
            string outputPath = txtOutputPath.Text;

            if (File.Exists(outputPath))
            {
                DialogResult dr = MessageBox.Show("Output file" + Environment.NewLine + Environment.NewLine + outputPath +
                    Environment.NewLine + Environment.NewLine + "exists. Overwrite it?", "Warning", MessageBoxButtons.YesNo);

                if (dr != DialogResult.Yes) return;
            }

            string template = null;

            using (Stream s = this.GetType().Assembly.GetManifestResourceStream("AltitudeShaderGen.SurfaceLayer.txt"))
            {
                using (StreamReader sr = new StreamReader(s))
                {
                    template = sr.ReadToEnd();
                }
            }
            
            List<LevelControl> lcs = new List<LevelControl>();
            foreach (object o in flowLayoutPanel.Controls)
            {
                lcs.Add(o as LevelControl);
            }

            StringBuilder tgc = new StringBuilder();

            tgc.AppendLine("<terragen_clip>");
            
            string slParent = template;
            slParent = slParent.Replace("%Name%", SL_BASE_NAME + "Parent");
            slParent = slParent.Replace("%InputNode%", string.Empty).Replace("%ChildLayers%", SL_BASE_NAME + (flowLayoutPanel.Controls.Count - 1));
            slParent = slParent.Replace("%MaximumAltitude%", string.Empty).Replace("%MaximumAltitudeFuzzy%", string.Empty);
            slParent = slParent.Replace("%MinimumAltitude%", string.Empty).Replace("%MinimumAltitudeFuzzy%", string.Empty);
            slParent = slParent.Replace("%ApplyColor%", "0");
            slParent = slParent.Replace("%DiffuseColor%", TGColor());
            tgc.AppendLine(slParent);

            var minToMax = (from lc in lcs orderby lc.Minimum ascending select lc);

            int level = 0;
            foreach (LevelControl lc in minToMax)
            {
                string sl = template;
                sl = sl.Replace("%Name%", SL_BASE_NAME + level);
                sl = sl.Replace("%InputNode%", DetermineInputNode(level));
                sl = sl.Replace("%ChildLayers%", "");
                sl = sl.Replace("%MaximumAltitude%", (lc.Maximum.HasValue ? lc.Maximum.Value.ToString() : string.Empty));
                sl = sl.Replace("%MaximumAltitudeFuzzy%", (lc.MaximumFuzzy.HasValue ? lc.MaximumFuzzy.Value.ToString() : string.Empty));
                sl = sl.Replace("%LimitMaximumAltitude%", (lc.Maximum.HasValue ? "1" : "0"));
                
                sl = sl.Replace("%MinimumAltitude%", (lc.Minimum.HasValue ? lc.Minimum.Value.ToString() : string.Empty));
                sl = sl.Replace("%MinimumAltitudeFuzzy%", (lc.MinimumFuzzy.HasValue ? lc.MinimumFuzzy.Value.ToString() : string.Empty));
                sl = sl.Replace("%LimitMinimumAltitude%", (lc.Minimum.HasValue ? "1" : "0"));

                sl = sl.Replace("%ApplyColor%", "1");
                sl = sl.Replace("%DiffuseColor%", TGColor(lc.SelectedColor));
                tgc.AppendLine(sl);

                level++;
            }

            tgc.AppendLine("</terragen_clip>");

            try
            {
                File.WriteAllText(outputPath, tgc.ToString());
                MessageBox.Show("Done!", "STATUS");
            }
            catch (Exception ex)
            {
                MessageBox.Show("Error writing the output file: " + Environment.NewLine + Environment.NewLine + ex.Message, "ERROR");
            }
        }

        private string TGColor()
        {
            return TGColor(Color.Gray);
        }

        private string TGColor(Color color)
        {
            double r = (double)color.R / 255.0;
            double g = (double)color.G / 255.0;
            double b = (double)color.B / 255.0;

            return r.ToString("0.00") + " " + g.ToString("0.00") + " " + b.ToString("0.00");
        }

        private string DetermineInputNode(int level)
        {
            if (level == 0)
            {
                return string.Empty;
            }
            else
            {
                return "/" + SL_BASE_NAME + (level - 1);
            }
        }

        private void btnClose_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void btnSave_Click(object sender, EventArgs e)
        {
            OpenFileDialog ofd = new OpenFileDialog();

            ofd.AddExtension = true;
            ofd.AutoUpgradeEnabled = true;
            ofd.DefaultExt = ".asgen";
            ofd.Filter = "Altitude Shader Gen Preset|*.asgen";
            ofd.CheckFileExists = false;

            DialogResult dr = ofd.ShowDialog();

            if (dr == DialogResult.OK)
            {
                if (File.Exists(ofd.FileName))
                {
                    DialogResult drOver = MessageBox.Show("File already exists.  Overwrite?", "WARNING", MessageBoxButtons.YesNo);
                    if (drOver == DialogResult.No) return;
                }

                List<LevelControlProperties> lcps = new List<LevelControlProperties>();
                foreach (object o in flowLayoutPanel.Controls)
                {
                    LevelControlProperties lcp = ((LevelControl)o).ControlProperties();
                    lcps.Add(lcp);
                }

                try
                {
                    using (Stream stream = File.Open(ofd.FileName, FileMode.Create))
                    {
                        BinaryFormatter bin = new BinaryFormatter();
                        bin.Serialize(stream, lcps);
                    }
                }
                catch (IOException ioe)
                {
                    MessageBox.Show("Could not save file: " + ioe.Message);
                }
            }
        }

        private void btnLoad_Click(object sender, EventArgs e)
        {
            OpenFileDialog ofd = new OpenFileDialog();

            ofd.AddExtension = true;
            ofd.AutoUpgradeEnabled = true;
            ofd.DefaultExt = ".asgen";
            ofd.Filter = "Alt. Shader Gen Preset|*.asgen";

            DialogResult dr = ofd.ShowDialog();

            if (dr != DialogResult.OK)
            {
                return;
            }

            if (!File.Exists(ofd.FileName))
            {
                DialogResult drOver = MessageBox.Show("File not found", "ERROR", MessageBoxButtons.OK);
                return;
            }


            try
            {
                using (Stream stream = File.Open(ofd.FileName, FileMode.Open))
                {
                    BinaryFormatter bin = new BinaryFormatter();

                    var lcps = (List<LevelControlProperties>)bin.Deserialize(stream);
                    
                    flowLayoutPanel.Controls.Clear();
                    foreach (LevelControlProperties lcp in lcps)
                    {
                        LevelControl lc = new LevelControl(lcp);
                        AddLC(lc);
                    }
                }
            }
            catch (IOException ioe)
            {
                MessageBox.Show("Could not load file: " + ioe.Message);
            }
        }

        private void btnHelp_Click(object sender, EventArgs e)
        {
            FrmHelp hlp = new FrmHelp();

            hlp.ShowDialog();
        }
    }
}
