﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace AltitudeShaderGen
{    
    public partial class LevelControl : UserControl
    {
        public delegate void RemoveHandler(LevelControl levelControl);

        public event RemoveHandler Remove;

        public LevelControl()
        {
            InitializeComponent();
        }

        protected void OnRemove()
        {
            if (Remove != null) Remove(this);
        }

        private void btnPickColor_Click(object sender, EventArgs e)
        {
            DialogResult dr = colorDialog.ShowDialog();
            if (dr == DialogResult.OK)
            {
                btnPickColor.BackColor = colorDialog.Color;
            }
        }

        private void btnRemove_Click(object sender, EventArgs e)
        {
            OnRemove();
        }

        public LevelControl(LevelControlProperties lcp) : this()
        {
            btnPickColor.BackColor = lcp.SelectedColor;
            txtMax.Text = NullableDoubleToString(lcp.Maximum);
            txtMaxFuzzy.Text = NullableDoubleToString(lcp.MaximumFuzzy);
            txtMin.Text = NullableDoubleToString(lcp.Minimum);
            txtMinFuzzy.Text = NullableDoubleToString(lcp.MinimumFuzzy);
        }

        public LevelControlProperties ControlProperties()
        {
            LevelControlProperties lcp = new LevelControlProperties();

            lcp.SelectedColor = SelectedColor;
            lcp.Maximum = Maximum;
            lcp.MaximumFuzzy = MaximumFuzzy;
            lcp.Minimum = Minimum;
            lcp.MinimumFuzzy = MinimumFuzzy;

            return lcp;
        }

        public Color SelectedColor
        {
            get { return btnPickColor.BackColor; }
        }

        public double? Maximum { get { return ParseNullableDouble(txtMax.Text); } }

        public double? MaximumFuzzy { get { return ParseNullableDouble(txtMaxFuzzy.Text); } }

        public double? Minimum { get { return ParseNullableDouble(txtMin.Text); } }

        public double? MinimumFuzzy { get { return ParseNullableDouble(txtMinFuzzy.Text); } }


        private string NullableDoubleToString(double? nullable)
        {
            if (nullable.HasValue)
            {
                return nullable.Value.ToString();
            }
            else
            {
                return string.Empty;
            }
        }

        private double? ParseNullableDouble(string input)
        {
            bool ok;
            double result;

            ok = double.TryParse(input, out result);

            if (ok)
            {
                return result;
            }
            else
            {
                return null;
            }
        }
    }
}
